//======================================================= file = common.h =====
//=  Include file for VOQ switch simulation model                             =
//=============================================================================
//=  Notes:  1) This file contains                                            =
//=               - Constants                                                 =
//=               - Common (external) variables                               =
//=               - Common (external) CSIM types                              =
//=               - Function prototypes                                       =
//=          2)  Basic time unit is the cell transmission time                =
//=---------------------------------------------------------------------------=
//=  History:  KJC (11/15/00) - Genesis                                       =
//=============================================================================
#pragma warn -pro                     // Disable function prototype warning
#pragma warn -stu                     // Disable undefined structure warning
#pragma warn -rch                     // Disable unreachable code warning
#pragma warn -ccc                     // Disable condition always true warning

//----- Conditional definitions -----------------------------------------------
#if defined(MAIN)                    // Determine if main or external
  #define EXTERN
#endif
#if defined(EXTERNAL)
  #define EXTERN extern
#endif

//----- Constants -------------------------------------------------------------
#define VERSION "1.02c <11/15/00>"    // Version number
#define OUT_FILE     "result.txt"     // Name of output file

#define FALSE                  0      // FALSE = 0
#define TRUE                   1      // TRUE = 1
#define PIM                    1      // PIM arbitration
#define ISLIP                  2      // ISLIP arbitration
#define PPVOQ                  3      // PP-VOQ arbitration

#define CELL_TIME            1.0      // Time to transmit a cell
#define N                     16      // Number of input ports in the switch
#define NUM_ITER               4      // Number of PIM or iSLIP iterations
#define ARBIT_TYPE         ISLIP      // Defines arbitration type (PIM, ISLIP,
                                      //   and PPVOQ are valid)
#define GEN_FLAG               1      // 1 for Bernoulli, 2 for IBP

//----- Overall external variables --------------------------------------------
EXTERN double    Sim_time;            // Simuation time
EXTERN int       Inport_count[N];     // Assertable inport counter semaphore
EXTERN int       Outport_count[N];    // Assertable outport counter semaphore

//----- External variables for generate1() ------------------------------------
EXTERN double    Lambda;              // Probability of a cell in a slot

//----- External variables for generate2() ------------------------------------
EXTERN double    P;                   // Pr[on at t+1 | on at t]
EXTERN double    Q;                   // Pr[off at t+1 | off at t]
EXTERN double    Alpha;               // Pr[arrival at t | on]

//----- CSIM types ------------------------------------------------------------
EXTERN EVENT    Q_event[N][N];        // Queueing event at inport
EXTERN EVENT    Inport_busy[N];       // Event for inport busy
EXTERN EVENT    Outport_busy[N];      // Event for outport busy
EXTERN EVENT    Xmit_done[N];         // Event for transmit done at outport
EXTERN TABLE    Delay_table[N];       // Delay table for cell delay

//----- Function prototypes ---------------------------------------------------
// Traffic generators
EXTERN void generate1(int in_id);
EXTERN void generate2(int in_id);
EXTERN void (* gen[N])(int in_id);

// Input ports with VOQ
EXTERN void inport(int in_id, int out_id, double org_time);
EXTERN void (* in[N])(int in_id, int out_id, double org_time);

// Output ports
EXTERN void outport(int from_id, int out_id, double org_time);
EXTERN void (* out[N])(int from_id, int out_id, double org_time);

// Switch arbitrators for PIM, iSLIP, and PP-VOQ
EXTERN void arbit_pim(void);
EXTERN void arbit_islip(void);
EXTERN void arbit_ppvoq(void);

// Run-time simulation status reporter
EXTERN void run_sim(double run_time, int run_num);
