//======================================================= file = inport.c =====
//=  VOQ input port for VOQ switch simulation model                           =
//=============================================================================
//=  Notes: 1) Asserts are used to insure that no illegal "double             =
//=            transmission" occurs                                           =
//=---------------------------------------------------------------------------=
//=  Contains: inport()                                                       =
//=---------------------------------------------------------------------------=
//=  History:  KJC (11/15/00) - Genesis                                       =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include <assert.h>     // Needed for assert() macro
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff

//============================================================================
//==  Input port for VOQ switch simulation model                            ==
//============================================================================
void inport(int in_id, int out_id, double org_time)
{
  create("inport");

  // Queue the cell in the appropriate VOQ (out_id) in this port (in_id)
  queue(Q_event[in_id][out_id]);

  // Assert that only one transmission occurs at an inport and outport
  assert(Inport_count[in_id] == 0);
  assert(Outport_count[out_id] == 0);

  // Increment counter semphores for an inport and outport pair
  Inport_count[in_id]++;
  Outport_count[out_id]++;

  // Forward the cell to the outport
  hold(CELL_TIME);
  out[out_id](in_id, out_id, org_time);

  return;
}
